/*
 * Copyright (C) Envisional Software Solutions Limited., 2010.
 * All rights reserved.
 *
 * Development for provision of Envisional Software Solutions
 * Service.
 */
package com.envisional.test;

import com.envisional.test.stuff.Report;
import com.envisional.test.stuff.Column;
import com.envisional.test.stuff.ResultsDb;
import com.envisional.test.stuff.ExcelExporter;
import com.envisional.test.stuff.Asynchronous;
import com.envisional.test.stuff.Persistence;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

public abstract class ResultsExporter<T> {

	protected Persistence db;
	protected EntityManager em;
	protected ExcelExporter excelExporter;
	protected List<Column<T>> columns;
	Report report = null;
	
	public ResultsExporter() {
	}

	public void setEntityManager(EntityManager em) {
		this.em = em;
		db = ResultsDb.getInstance(this.em);
	}

	private void createExcelExporter(List<Column<T>> columns) throws Exception {
		excelExporter = new ExcelExporter();
		this.columns = columns;
		List<Object> headings = new ArrayList<Object>();
		for (Column c : columns) {
			if (c.getDefinition()) {
				headings.add(c.getTitle());
			}
		}
		excelExporter.addRow(headings);
	}

	protected abstract void addRows(Asynchronous<T> a) throws Exception;
	public abstract void postProcess() throws Exception;

	private void exportResults(List<Column<T>> columns, Asynchronous<T> a) throws Exception {
		createExcelExporter(columns);
		addRows(a);
	}

	protected void addRow(List<Object> row) throws Exception {
		excelExporter.addRow(row);
	}

	public void exportResultsToFile(List<Column<T>> columns, String location, Asynchronous<T> a) throws Exception {
		exportResults(columns, a);
		excelExporter.exportToFile(location);
	}

	public byte[] exportResultsAsBytes(List<Column<T>> columns, Asynchronous<T> a) throws Exception {
		exportResults(columns, a);
		return excelExporter.exportBytes();
	}

	public void setReport(Report report) {
		this.report = report;
	}

	public Report getReport() {
		return report;
	}
}
